#!/usr/bin/perl
#
# Create MMU initialization file
#

# Each MMU entry is 16 bits; there are 4 address maps total.
# These are initialized with a 2K video RAM window at 30-32K.
# The defaults match the map uses from Mikrodatorn's 64K RAM
# expansion:
#
# Map 0: default (32K RAM)
# Map 1: 0-30K is RAM
# Map 2: 0-62K RAM; 62-64K video
# Map 3: 0-64K RAM, no video
#
# The format of each entry is as follows:
# Bits [15:12] -> device index:
#                 0 - SRAM
#                 1 - Internal test RAM (4K)
#                 2 - Video RAM
#                 3 - Character generator RAM
#                 4 - BASIC ROM
#                 5 - DOS ROM
#                 F - Hyperspace (reads FF, writes nowhere)
#                 The rest reserved for Ethernet, Flash,...
# Bits [11:0] -> A[19:8]
#

$testram = 0;

sub do_ram() {
    if ( $testram ) {
	for ( ; $i < 0xF0 ; $i++ ) {
	    printf("%03X : %04X;\n", $a++, 0xFFFF);
	}
	for ( $i = 0xF0 ; $i < 0x100 ; $i++ ) {
	    printf("%03X : %04X;\n", $a++, 0x1000|($i & 0x0F));
	}
    } else {
	for ( ; $i < 0x100 ; $i++ ) {
	    # Important: No offset for this index
	    printf("%03X : %04X;\n", $a++, 0x0000|$i);
	}
    }
}

print "DEPTH = 1024;\n";
print "WIDTH = 16;\n";
print "ADDRESS_RADIX = HEX;\n";
print "DATA_RADIX    = HEX;\n";
print "CONTENT\n";
print "BEGIN\n";

$a = 0;

# Map 0
for ( $i = 0x00 ; $i < 0x40 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x4000|$i);
}
for ( $i = 0x40 ; $i < 0x60 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0xFFFF);
}
for ( $i = 0x60 ; $i < 0x70 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x5000|($i-0x60));
}
for ( $i = 0x70 ; $i < 0x78 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0xFFFF);
}
for ( $i = 0x78 ; $i < 0x80 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x2000|($i-0x78));
}
do_ram();

# Map 1
for ( $i = 0x00 ; $i < 0x78 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x0000|$i);
}
for ( $i = 0x78 ; $i < 0x80 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x2000|($i-0x78));
}
do_ram();

# Map 2
for ( $i = 0x00 ; $i < 0xF8 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x0000|$i);
}
for ( $i = 0xF8 ; $i < 0x100 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x2000|($i-0xF8));
}

# Map 3
for ( $i = 0x00 ; $i < 0x100 ; $i++ ) {
    printf("%03X : %04X;\n", $a++, 0x0000|$i);
}

print "END;\n";
